@@ -2,7 +2,7 @@ package Net::Rendezvous;
 
 =head1 NAME
 
-Net::Rendezvous - Module for mDNS service discovery (Apple's Rendezvous)
+Net::Rendezvous - Module for DNS service discovery (Apple's Rendezvous)
 
 =head1 SYNOPSIS 
 
@@ -10,6 +10,8 @@ Net::Rendezvous - Module for mDNS service discovery (Apple's Rendezvous)
 	
 	my $res = Net::Rendezvous->new(<service>[, <protocol>]);
 
+	$res->discover;
+
 	foreach my $entry ( $res->entries ) {
 		printf "%s %s:%s\n", $entry->name, $entry->address, $entry->port;
 	}
@@ -20,21 +22,24 @@ Or the cyclical way:
 
 	my $res = Net::Rendezvous->new(<service>[, <protocol>]);
                
+	$res->discover;
+
 	while ( 1 ) {
 	   foreach my $entry ( $res->entries ) {
 		   print $entry->name, "\n";
 	   }
-	   $res->refresh;
+	   $res->discover;
    	}
 
 =head1 DESCRIPTION
 
-Net::Rendezvous is a set of modules that allow one to discover local services via multicast DNS (mDNS).
-This method of service discovery has been branded as Rendezvous by Apple Computer.
+Net::Rendezvous is a set of modules that allow one to discover local services via multicast DNS (mDNS) 
+or enterprise services via traditional DNS.  This method of service discovery has been branded as 
+Rendezvous by Apple Computer.
 
 =head2 Base Object
 
-The base object would be of the Net::Rendezvous class.  This object contains the resolver for mDNS service discovery.
+The base object would be of the Net::Rendezvous class.  This object contains the resolver for DNS service discovery.
 
 =head2 Entry Object
 
@@ -42,16 +47,21 @@ The base object (Net::Rendezvous) will return entry objects of the class L<Net::
 
 =head1 METHODS
 
-=head2 new([<service>, <protocol>])
+=head2 new([<service>, <protocol>, <domain>])
 
 Creates a new Net::Rendezvous discovery object.  First argument specifies the service to discover, 
 e.g.  http, ftp, afpovertcp, and ssh.  The second argument specifies the protocol, i.e. tcp or udp.  
-I<The default protocol is TCP>.  
+I<The default protocol is TCP>. The third argument specifies the discovery domain, the default is 'local'. 
 
 If no argments are specified, the resulting Net::Rendezvous object will be empty and will not perform an 
 automatic discovery upon creation.
 
-=head2 refresh
+=head2 domain([<domain>])
+
+Get/sets current discovery domain.  By default, the discovery domain is 'local'.  Discovery for the 'local'
+domain is done via MDNS while all other domains will be done via traditional DNS.
+
+=head2 discover
 
 Repeats the discovery process and reloads the entry list from this discovery.
 
@@ -72,6 +82,7 @@ Shifts off the first entry of the last discovery.  The returned object will be a
         use Net::Rendezvous;
 
         my $res = Net::Rendezvous->new('http');
+	$res->discover;
 
         foreach my $entry ( $res->entries) {
                 printf "<A HREF='http://%s%s'>%s</A><BR>", $entry->address, 
@@ -86,6 +97,7 @@ Shifts off the first entry of the last discovery.  The returned object will be a
 	use Net::Rendezvous;
         
         my $res = Net::Rendezvous->new('custom');
+	$res->discover;
         
         my $entry = $res->shift_entry;
         
@@ -122,7 +134,7 @@ use Net::DNS;
 use Net::Rendezvous::Entry;
 use Socket;
 
-$VERSION = 0.70;
+$VERSION = '0.80';
 
 sub new {
 	my $self = {};
@@ -134,12 +146,14 @@ sub new {
 sub _init {
 	my $self = shift;
 
-	$self->{'_dns_server'} = '224.0.0.251';
+	$self->{'_dns_server'} = [ '224.0.0.251' ];
 	$self->{'_dns_port'} = '5353';
+	$self->{'_dns_domain'} = 'local';
 
 	if (@_) {
-		$self->application(shift);
-		$self->refresh;
+		$self->domain(pop) if $_[$#_] =~ /\./;
+		$self->application(@_);
+		$self->discover;
 	}
 	return;
 }
@@ -150,20 +164,42 @@ sub application {
 	if (@_) {
 		my $app = shift;
 		my $proto = shift || 'tcp';
-		$self->{'_app'} = sprintf '_%s._%s.local', $app, $proto;
+		$self->{'_app'} = sprintf '_%s._%s.%s', $app, $proto, 
+			$self->{'_dns_domain'};
 	}
 
 	return $self->{'_app'};
 }
 
-sub refresh {
+sub dns_refresh { 
+	my $self = shift;
+	
+	my $resolv = Net::DNS::Resolver->new();
+	
+	my $query = $resolv->query($self->application, 'PTR');
+	my $list = [];
+
+	foreach my $rr ($query->answer) {
+		next if $rr->type ne 'PTR';
+		my $host = Net::Rendezvous::Entry->new($rr->ptrdname);
+		$host->dns_server($resolv->nameservers);
+		$host->dns_port($resolv->port);
+		$host->fetch;
+		push(@{$list}, $host);
+	}
+
+	$self->{'_results'} = $list;
+	return $#{$list};
+}
+
+sub mdns_refresh {
 	my $self = shift;
 
 	my $query = Net::DNS::Packet->new($self->application, 'PTR');
 
 	socket DNS, PF_INET, SOCK_DGRAM, scalar(getprotobyname('udp'));
 	bind DNS, sockaddr_in(0,inet_aton('0.0.0.0'));
-	send DNS, $query->data, 0, sockaddr_in($self->{'_dns_port'}, inet_aton($self->{'_dns_server'}));
+	send DNS, $query->data, 0, sockaddr_in($self->{'_dns_port'}, inet_aton($self->{'_dns_server'}[0]));
 
 	my $rout = '';
 	my $rin  = '';
@@ -177,12 +213,14 @@ sub refresh {
 		recv(DNS, $data, 1000, 0);
 
 		my $ans = Net::DNS::Packet->new(\$data);
+		next if $query->header->id != $ans->header->id;
 
 		foreach my $rr ($ans->answer) {
-			my $host = Net::Rendezvous::Entry->new($rr->rdatastr);
+			next if $rr->type ne 'PTR';
+			my $host = Net::Rendezvous::Entry->new($rr->ptrdname);
 			$host->dns_server($self->{'_dns_server'});
 			$host->dns_port($self->{'_dns_port'});
-			$host->fetch($rr->rdatastr);
+			$host->fetch;
 			push(@{$list}, $host);
 		}
 	}
@@ -201,4 +239,30 @@ sub shift_entry {
 	return shift(@{$self->{'_results'}});
 }
 
+sub domain {
+	my $self = shift;
+	
+	if ( @_ ) {
+		$self->{'_dns_domain'} = shift;
+		$self->{'_dns_domain'} =~ s/[(^\.)(\.$)]//;
+	}
+	return $self->{'_dns_domain'};
+}
+
+sub refresh { 
+	my $self = shift;
+	return $self->discover(@_);
+}
+
+sub discover {
+	my $self = shift;
+
+	if ( $self->domain(@_) eq 'local' ) {
+		return $self->mdns_refresh;
+	} else {
+		return $self->dns_refresh;
+	}
+
+	return;
+}
 1;
